﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2011 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Drawing.Text;
using System.Windows.Forms;
using HeuristicLab.Common;
using HeuristicLab.Data;
using HeuristicLab.MainForm;
using HeuristicLab.MainForm.WindowsForms;

namespace HeuristicLab.Analysis.Views {
  [View("Coordinates View")]
  [Content(typeof(DoubleMatrix), IsDefaultView = false)]
  public partial class CoordinatesView : AsynchronousContentView {
    private Bitmap bitmap;

    public new DoubleMatrix Content {
      get { return (DoubleMatrix)base.Content; }
      set { base.Content = value; }
    }

    public CoordinatesView() {
      InitializeComponent();
    }

    #region Register Content Events
    protected override void DeregisterContentEvents() {
      Content.Reset -= new EventHandler(Content_Reset);
      Content.ItemChanged -= new EventHandler<EventArgs<int, int>>(Content_Reset);
      base.DeregisterContentEvents();
    }
    protected override void RegisterContentEvents() {
      base.RegisterContentEvents();
      Content.Reset += new EventHandler(Content_Reset);
      Content.ItemChanged += new EventHandler<EventArgs<int, int>>(Content_Reset);
    }

    void Content_Reset(object sender, EventArgs<int, int> e) {
      GenerateImage();
    }

    void Content_Reset(object sender, EventArgs e) {
      GenerateImage();
    }
    #endregion

    protected override void OnContentChanged() {
      base.OnContentChanged();
      if (Content == null) {
        pictureBox.Image = null;
      } else {
        GenerateImage();
      }
    }

    protected override void SetEnabledStateOfControls() {
      base.SetEnabledStateOfControls();
      pictureBox.Enabled = Content != null;
    }

    private void GenerateImage() {
      DoubleMatrix coordinates = Content;
      if (coordinates == null || coordinates.Rows < 1 || coordinates.Columns != 2) return;

      Bitmap newBitmap = new Bitmap(pictureBox.Width, pictureBox.Height);
      double xMin = double.MaxValue, yMin = double.MaxValue, xMax = double.MinValue, yMax = double.MinValue;
      for (int i = 0; i < coordinates.Rows; i++) {
        if (xMin > coordinates[i, 0]) xMin = coordinates[i, 0];
        if (yMin > coordinates[i, 1]) yMin = coordinates[i, 1];
        if (xMax < coordinates[i, 0]) xMax = coordinates[i, 0];
        if (yMax < coordinates[i, 1]) yMax = coordinates[i, 1];
      }

      int border = 20;
      double xStep = xMax != xMin ? (pictureBox.Width - 2 * border) / (xMax - xMin) : 1;
      double yStep = yMax != yMin ? (pictureBox.Height - 2 * border) / (yMax - yMin) : 1;

      Point[] points = new Point[coordinates.Rows];
      for (int i = 0; i < coordinates.Rows; i++)
        points[i] = new Point(border + ((int)((coordinates[i, 0] - xMin) * xStep)),
                              newBitmap.Height - (border + ((int)((coordinates[i, 1] - yMin) * yStep))));

      using (Graphics graphics = Graphics.FromImage(newBitmap)) {
        graphics.SmoothingMode = SmoothingMode.AntiAlias;
        graphics.TextRenderingHint = TextRenderingHint.AntiAlias;

        for (int i = 0; i < points.Length; i++) {
          Point p = new Point(points[i].X - 3, points[i].Y - 3);
          graphics.FillRectangle(Brushes.Black, p.X, p.Y, 8, 8);
          graphics.DrawString(i.ToString(), Font, Brushes.Black, p.X, p.Y + 10);
        }
      }

      pictureBox.Image = newBitmap;
      if (bitmap != null) bitmap.Dispose();
      if (newBitmap != null) bitmap = newBitmap;
      else bitmap = null;
    }

    private void pictureBox_SizeChanged(object sender, System.EventArgs e) {
      GenerateImage();
    }

    #region Event Handlers
    // TODO: Put event handlers here
    #endregion
  }
}
